#!/usr/bin/python3
# Copyright (C) 2017  Lars Wirzenius
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os

import apifw
import slog
import yaml

import salami


DEFAULT_CONFIG_FILE = '/dev/null'


def dict_logger(log, stack_info=None):
    salami.log.log(exc_info=stack_info, **log)


def read_config(filename):
    with open(filename) as f:
        return yaml.safe_load(f)


def check_config(cfg):
    for key in cfg:
        if cfg[key] is None:
            raise Exception('Configration %s should not be None' % key)


_counter = slog.Counter()


def counter():
    new_context = 'HTTP transaction {}'.format(_counter.increment())
    salami.log.set_context(new_context)


default_config = {
    'log': [],
}


config_filename = os.environ.get('SALAMI_CONFIG', DEFAULT_CONFIG_FILE)
actual_config = read_config(config_filename)
config = dict(default_config)
config.update(actual_config or {})
check_config(config)
salami.setup_logging(config)
salami.log.log('info', msg_text='Salami starting')

api = salami.SalamiAPI()
app = apifw.create_bottle_application(api, counter, dict_logger, config)

# If we are running this program directly with Python, and not via
# gunicorn, we can use the Bottle built-in debug server, which can
# make some things easier to debug.

if __name__ == '__main__':
    print('running in debug mode')
    app.run(host='127.0.0.1', port=12765)
